@@ -100,7 +100,37 @@ pub fn del<
     //
     // We delete that position, which might be in a leaf or in an
     // internal node.
-    del_at_cursor(txn, db, &mut cursor)
+    del_at_cursor(txn, db, &mut cursor, true)
+}
+
+
+/// If `value` is `None`, delete the first entry for `key` from the
+/// database, if present. Else, delete the entry for `(key, value)`,
+/// if present.
+pub fn del_no_decr<
+    T: AllocPage + LoadPage,
+    K: Storable + ?Sized + PartialEq,
+    V: Storable + ?Sized + PartialEq,
+    P: BTreeMutPage<K, V>,
+>(
+    txn: &mut T,
+    db: &mut Db_<K, V, P>,
+    key: &K,
+    value: Option<&V>,
+) -> Result<bool, T::Error> {
+    let mut cursor = Cursor::new(txn, db)?;
+    // If the key and value weren't found, return.
+    match (cursor.set(txn, key, value)?, value) {
+        (Some((k, v)), Some(value)) if k == key && v == value => {}
+        (Some((k, _)), None) if k == key => {}
+        _ => return Ok(false),
+    }
+    // Else, the cursor is set on `(key, value)` if `value.is_some()`
+    // or on the first entry with key `key` else.
+    //
+    // We delete that position, which might be in a leaf or in an
+    // internal node.
+    del_at_cursor(txn, db, &mut cursor, false)
 }
 
 /// Delete the entry pointed to by `cursor`. The cursor can't be used
@@ -115,6 +145,7 @@ pub fn del_at_cursor<
     txn: &mut T,
     db: &mut Db_<K, V, P>,
     cursor: &mut Cursor<K, V, P>,
+    decr_del_rc: bool,
 ) -> Result<bool, T::Error> {
     let p0 = cursor.len();
 
@@ -126,14 +157,17 @@ pub fn del_at_cursor<
     // Else, that RC doesn't need to be changed, since it's still
     // referenced by the same pages as before, just not by the pages
     // that are cloned by this deletion.
-    if p0 < cursor.first_rc_len() {
+    //
+    // Note in particular that if p0 == cursor.first_rc_len(), then we
+    // don't need to touch the RC, since we're cloning this page, but
+    // without including a reference to the deleted entry.
+    if decr_del_rc && p0 < cursor.first_rc_len() {
         let (delk, delv, _) = {
             let cur = cursor.cur();
             P::current(txn, cur.page.as_page(), &cur.cursor).unwrap()
         };
-        for o in delk.page_references().chain(delv.page_references()) {
-            txn.decr_rc(o)?;
-        }
+        delk.drop(txn)?;
+        delv.drop(txn)?;
     }
 
     // Find the leftmost page in the right subtree of the deleted
