@@ -17,7 +17,7 @@ use crate::*;
 pub mod cursor;
 pub use cursor::{iter, rev_iter, Cursor, Iter, RevIter};
 pub mod del;
-pub use del::del;
+pub use del::{del, del_no_decr};
 pub mod put;
 pub use put::put;
 pub mod page;
@@ -247,6 +247,17 @@ pub struct Db_<K: ?Sized, V: ?Sized, P: BTreePage<K, V>> {
 /// A database of sized values.
 pub type Db<K, V> = Db_<K, V, page::Page<K, V>>;
 
+impl<K:Storable, V:Storable, P: BTreePage<K, V> + core::fmt::Debug> Storable for Db_<K, V, P> {
+    type PageReferences = core::iter::Once<u64>;
+    fn page_references(&self) -> Self::PageReferences {
+        core::iter::once(self.db)
+    }
+
+    fn drop<T: AllocPage>(&self, t: &mut T) -> Result<(), T::Error> {
+        drop_(t, self)
+    }
+}
+
 /// A database of unsized values.
 pub type UDb<K, V> = Db_<K, V, page_unsized::Page<K, V>>;
 
@@ -298,13 +309,66 @@ pub fn fork_db<T: AllocPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTre
 }
 
 /// Get the first entry of a database greater than or equal to `k` (or
-/// to `(k, v)` if `v.is_some()`).
+/// to `(k, v)` if `v.is_some()`), and return `true` if this entry is
+/// shared with another structure (i.e. the RC of at least one page
+/// along a path to the entry is at least 2).
+pub fn get_shared<'a, T: LoadPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTreePage<K, V>>(
+    txn: &'a T,
+    db: &Db_<K, V, P>,
+    k: &K,
+    v: Option<&V>,
+) -> Result<Option<(&'a K, &'a V, bool)>, T::Error> {
+
+    // Set the "cursor stack" by setting a skip list cursor in
+    // each page from the root to the appropriate leaf.
+    let mut last_match = None;
+    let mut page = txn.load_page(db.db)?;
+    let mut is_shared = txn.rc(db.db)? >= 2;
+    // This is a for loop, to allow the compiler to unroll (maybe).
+    for _ in 0..cursor::N_CURSORS {
+        let mut cursor = P::cursor_before(&page);
+        if let Ok((kk, vv, _)) = P::set_cursor(txn, page.as_page(), &mut cursor, k, v) {
+            if v.is_some() {
+                return Ok(Some((kk, vv, is_shared)));
+            }
+            last_match = Some((kk, vv, is_shared));
+        } else if let Some((k, v, _)) = P::current(txn, page.as_page(), &cursor) {
+            // Here, Rust says that `k` and `v` have the same lifetime
+            // as `page`, but we actually know that they're alive for
+            // as long as `txn` doesn't change, so we can safely
+            // extend their lifetimes:
+            unsafe { last_match = Some((core::mem::transmute(k), core::mem::transmute(v), is_shared)) }
+        }
+        // The cursor is set to the first element greater than or
+        // equal to the (k, v) we're looking for, so we need to
+        // explore the left subtree.
+        let next_page = P::left_child(page.as_page(), &cursor);
+        if next_page > 0 {
+            page = txn.load_page(next_page)?;
+            is_shared |= txn.rc(db.db)? >= 2;
+        } else {
+            break;
+        }
+    }
+    Ok(last_match)
+}
+
+
+/// Get the first entry of a database greater than or equal to `k` (or
+/// to `(k, v)` if `v.is_some()`), and return `true` if this entry is
+/// shared with another structure (i.e. the RC of at least one page
+/// along a path to the entry is at least 2).
 pub fn get<'a, T: LoadPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTreePage<K, V>>(
     txn: &'a T,
     db: &Db_<K, V, P>,
     k: &K,
     v: Option<&V>,
 ) -> Result<Option<(&'a K, &'a V)>, T::Error> {
+
+    // Unfortunately, since the above function `get_shared` uses this
+    // one in order to get the RC of pages, we can't really refactor,
+    // so this is mostly a copy of the other function.
+
     // Set the "cursor stack" by setting a skip list cursor in
     // each page from the root to the appropriate leaf.
     let mut last_match = None;
@@ -342,6 +406,13 @@ pub fn get<'a, T: LoadPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTree
 pub fn drop<T: AllocPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTreePage<K, V>>(
     txn: &mut T,
     db: Db_<K, V, P>,
+) -> Result<(), T::Error> {
+    drop_(txn, &db)
+}
+
+fn drop_<T: AllocPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTreePage<K, V>>(
+    txn: &mut T,
+    db: &Db_<K, V, P>,
 ) -> Result<(), T::Error> {
     // In order to make this function tail-recursive, we simulate a
     // stack with the following:
@@ -401,9 +472,8 @@ pub fn drop<T: AllocPage, K: Storable + ?Sized, V: Storable + ?Sized, P: BTreePa
             // if there's a current element in the cursor (i.e. we
             // aren't before or after the elements), decrease its RC.
             if let Some((k, v, _)) = P::current(txn, cur.page.as_page(), &cur.cursor) {
-                for o in k.page_references().chain(v.page_references()) {
-                    txn.decr_rc(o)?;
-                }
+                k.drop(txn)?;
+                v.drop(txn)?;
             }
             // In all cases, move next and push the left child onto
             // the stack. This works because pushed cursors are
